SystemOrganization addCategory: #'System-Download'!

Object subclass: #CodeLoader
	instanceVariableNames: 'sourceFiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Download'!

!CodeLoader commentStamp: '<historical>' prior: 0!
CodeLoader provides a way to load smalltalk source file from local or disant Url.

Example: 

CodeLoader new 
	loadSourceFiles: (Array with: 
		(#file:, 
		FileDirectory default fullName, 
		'/useless.st')); 
	installSourceFiles !

!CodeLoader class methodsFor: 'CommandLine Handling' stamp: 'AdrienBarreau 2/13/2011 15:28'!
commandLineHandlerAction: aCommandLine
	"Load the file specified on the command line"
	| url param loader |
	
	param := (aCommandLine parameterAt: 1) convertFromSystemString.
	url := Url absoluteFromText: param.
	(url class = HttpUrl and: [ (param asLowercase beginsWith: 'http:') not ]) ifTrue: [  "default url returned is http but here param is a local file"
		param := FileUrl schemeName, ':', (param first = $/ ifFalse: [ FileDirectory default fullName, '/' ] ifTrue: [ '' ]), param].
	
	loader := self new.
	loader loadSourceFiles: (Array with: param).
	loader installSourceFiles.! !

!CodeLoader class methodsFor: 'CommandLine Handling' stamp: 'AdrienBarreau 2/5/2011 19:49'!
commandLineHandlerCondition: aCommandLine
	"Ensure that:
		- there is a parameter
		- it could be cast to an Url (Ftp, Http or File protocols)"
	| url classes |

	url := Url absoluteFromText: ((aCommandLine parameterAt: 1) ifNil: [ String empty ]).
	classes := OrderedCollection with: FileUrl with: FtpUrl with: HttpUrl.
	
	^ (classes includes: url class) and: [ 
		(url class = (Url urlClassForScheme: #html)) ifTrue: [ url beginsWith: 'html:' ] ifFalse: [ true ] ]
	
	! !

!CodeLoader class methodsFor: 'CommandLine Handling' stamp: 'AdrienBarreau 2/5/2011 19:30'!
enableCommandLineHandling: aBoolean
	aBoolean 
		ifTrue: [ 
			[CommandLine register: [ :cl | CodeLoader commandLineHandlerAction: cl ] if: [ :cl | CodeLoader commandLineHandlerCondition: cl ] name: CodeLoader asString] on: Error do: []
		 ]
		ifFalse: [ CommandLine removeRegistrationNamed: CodeLoader asString ]! !

!CodeLoader class methodsFor: 'CommandLine Handling' stamp: 'AdrienBarreau 4/12/2011 11:27'!
initialize
	CommandLine register: [ :cl | CodeLoader commandLineHandlerAction: cl] if:[ :cl | CodeLoader commandLineHandlerCondition: cl] name: #CodeLoader! !

!CodeLoader methodsFor: 'private' stamp: 'AdrienBarreau 2/5/2011 22:45'!
createRequestFor: name in: aLoader
	"Create a URL request for the given string, which can be cached locally."
	| request |
	request := UrlCodeFileLoader httpRequestClass for: name in: aLoader.
	aLoader addRequest: request. "fetch from URL"
	^request! !

!CodeLoader methodsFor: 'private' stamp: 'AdrienBarreau 2/5/2011 21:48'!
httpRequestClass
	^UrlDownloadRequest! !

!CodeLoader methodsFor: 'installing' stamp: 'sd 1/30/2004 15:16'!
installSourceFile: aStream
	"Install the previously loaded source file"
	| contents trusted |
	aStream ifNil:[^self error:'No content to install'].
	trusted := SecurityManager default positionToSecureContentsOf: aStream.
	trusted ifFalse:[(SecurityManager default enterRestrictedMode) 
					ifFalse:[ aStream close.
							^ self error:'Insecure content encountered']].
	contents := aStream ascii upToEnd unzipped.
	(aStream respondsTo: #close) ifTrue:[aStream close].
	^(RWBinaryOrTextStream with: contents) reset fileIn! !

!CodeLoader methodsFor: 'installing' stamp: 'MarcusDenker 8/18/2010 11:17'!
installSourceFiles
	"Install the previously loaded source files"

	sourceFiles isNil
		ifTrue: [ ^ self ].
	sourceFiles do: [ :req | self installSourceFile: req contentStream ].
	sourceFiles := nil! !

!CodeLoader methodsFor: 'loading' stamp: 'AdrienBarreau 2/5/2011 21:48'!
loadSourceFiles: anArray
	"Load all the source files in the given array."
	| loader |
	loader := UrlCodeFileLoader default.
	sourceFiles := anArray collect:[:name| | request |
		request := self createRequestFor: name in: loader.
		request].
! !

Object subclass: #UrlCodeFileLoader
	instanceVariableNames: 'requests downloads'
	classVariableNames: 'DefaultLoader MaxNrOfConnections'
	poolDictionaries: ''
	category: 'System-Download'!

!UrlCodeFileLoader commentStamp: '<historical>' prior: 0!
That class is used by CodeLoader. It loads some files with their Urls.
For each file, a UrlDownloadRequest is created.!

!UrlCodeFileLoader classSide methodsFor: 'accessing' stamp: 'AdrienBarreau 2/5/2011 21:48'!
default
	DefaultLoader ifNil: [
		DefaultLoader := UrlCodeFileLoader new].
	^DefaultLoader! !

!UrlCodeFileLoader classSide methodsFor: 'accessing' stamp: 'AdrienBarreau 2/5/2011 21:48'!
httpRequestClass
		^UrlDownloadRequest! !

!UrlCodeFileLoader classSide methodsFor: 'initialization' stamp: 'mir 3/8/2001 16:31'!
initialize
	"HTTPLoader initialize"

	MaxNrOfConnections := 4.
	DefaultLoader ifNotNil: [
		DefaultLoader release.
		DefaultLoader := nil]! !

!UrlCodeFileLoader methodsFor: 'requests' stamp: 'MarcusDenker 12/8/2009 17:54'!
abort
	| oldRequests |
	"Abort all requests"
	oldRequests := requests.
	requests := SharedQueue new. 
	[oldRequests isEmpty] whileFalse: [
		oldRequests next signalAbort].
	downloads do: [:each | each ifNotNil: [each terminate]].
	downloads := OrderedCollection new
! !

!UrlCodeFileLoader methodsFor: 'private' stamp: 'AdrienBarreau 2/5/2011 21:33'!
addRequest: aHTTPRequest
	requests nextPut: aHTTPRequest.
	self startDownload! !

!UrlCodeFileLoader methodsFor: 'initialize/release' stamp: 'MarcusDenker 12/8/2009 17:53'!
initialize
	super initialize.
	requests := SharedQueue new.
	downloads := OrderedCollection new! !

!UrlCodeFileLoader methodsFor: 'private' stamp: 'mir 10/4/1999 18:31'!
maxNrOfConnections
	^MaxNrOfConnections! !

!UrlCodeFileLoader methodsFor: 'private' stamp: 'mir 10/7/1999 18:16'!
nextRequest
	^requests next! !

!UrlCodeFileLoader methodsFor: 'initialize/release' stamp: 'mir 10/7/1999 16:59'!
release
	self abort.
	downloads := nil.
	requests := nil! !

!UrlCodeFileLoader methodsFor: 'private' stamp: 'md 11/14/2003 16:38'!
removeProcess: downloadProcess
	downloads remove: downloadProcess ifAbsent: []! !

!UrlCodeFileLoader methodsFor: 'requests' stamp: 'mir 4/16/2001 17:48'!
retrieveContentsFor: url
	| request |
	request := self class httpRequestClass for: url in: self.
	self addRequest: request.
	^request contents! !

!UrlCodeFileLoader methodsFor: 'private' stamp: 'mir 5/12/2003 18:10'!
startDownload
	| newDownloadProcess |
	
	downloads size >= self maxNrOfConnections ifTrue: [^self].
	requests size <= 0 ifTrue: [^self].

	newDownloadProcess := [
		[
			self nextRequest startRetrieval
		] on: FTPConnectionException do: [ :ex | 
			Cursor normal show.
			self removeProcess: Processor activeProcess.
			self startDownload
		].
		self removeProcess: Processor activeProcess.
		self startDownload
	] newProcess.
	downloads add: newDownloadProcess.
	newDownloadProcess resume! !

Object subclass: #UrlDownloadRequest
	instanceVariableNames: 'semaphore url content loader process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Download'!

!UrlDownloadRequest commentStamp: '<historical>' prior: 0!
That class is used by UrlCodeFileLoader; it represents a request for the downloading of a single file.
It is that object which has to download the file and it can then provide its content.!

!UrlDownloadRequest classSide methodsFor: 'instance creation' stamp: 'mir 10/7/1999 16:59'!
for: aUrl in: aLoader
	^self new for: aUrl in: aLoader! !

!UrlDownloadRequest methodsFor: 'accessing' stamp: 'mir 10/4/1999 18:26'!
content: retrievedContent
	content := retrievedContent.
	semaphore signal! !

!UrlDownloadRequest methodsFor: 'accessing' stamp: 'ar 5/30/2001 21:03'!
contentStream
	"Return a stream on the content of a previously completed HTTP request"
	semaphore wait.
	^content ifNotNil:[content contentStream]! !

!UrlDownloadRequest methodsFor: 'accessing' stamp: 'mir 10/4/1999 18:25'!
contents
	semaphore wait.
	^content! !

!UrlDownloadRequest methodsFor: 'initialize' stamp: 'KR 9/4/2005 10:33'!
for: aUrl in: aLoader
	url := aUrl.
	loader := aLoader.
	semaphore := Semaphore new.! !

!UrlDownloadRequest methodsFor: 'accessing' stamp: 'mir 10/7/1999 16:57'!
process: aProcess
	process := aProcess! !

!UrlDownloadRequest methodsFor: 'accessing' stamp: 'mir 1/15/2000 22:55'!
signalAbort
	loader removeProcess: process.
	self content: 'Retrieval aborted'.
	process ifNotNil: [process terminate]! !

!UrlDownloadRequest methodsFor: 'accessing' stamp: 'AdrienBarreau 2/5/2011 21:16'!
startRetrieval
	self content: url asUrl retrieveContents! !

!UrlDownloadRequest methodsFor: 'accessing' stamp: 'mir 1/20/2000 13:33'!
url
	^url! !
CodeLoader initialize!
UrlCodeFileLoader initialize!
